import sys
import os
import time
i, o, e = sys.stdin, sys.stdout, sys.stderr
from scapy.all import *
__author__ = 'jacob shutzman'
"""
   Implementation of trace route, using IP and ICMP protocols
   via the scapy module. Using increasing TTL's until we reach
   the destination, and writing back the hops and times it took
"""
sys.stdin, sys.stdout, sys.stderr = i, o, e

# Constants
TTL_EXPIRED_TYPE = 11
IN_TRANSIT_CODE = 0
ECHO_REPLY_TYPE = 0
MY_TIMEOUT = 5


def main():
    if len(sys.argv) != 2:
        print 'Usage: my_tracert.py url'
        sys.exit(1)

    # we start with 1
    ttl = 1
    end_check = False
    while not end_check and ttl < 30:  # Maximum hops: 30
        hop_start_t = time.time()
        try:
            response = sr1(IP(dst=sys.argv[1], ttl=ttl)/ICMP(),
                           verbose=0, timeout=MY_TIMEOUT)
        except:
            print 'Unresolved URL'
            sys.exit(1)
        hop_end_t = time.time()
        # TTL exceeded
        if response:
            # Convert hop time to milliseconds for display
            hop_time = int((hop_end_t - hop_start_t)*1000)
            if response[ICMP].type == TTL_EXPIRED_TYPE and \
               response[ICMP].code == IN_TRANSIT_CODE:
                print '{0:>4} ms {1:>2} -> {2:<16}'. \
                      format(str(hop_time), ttl, str(response.src))
                # print str(hop_time)+' '+'ms', ttl, '->', response.src
                ttl += 1
            elif response[ICMP].type == ECHO_REPLY_TYPE:
                print '{0:>4} ms {1:>2} -> {2:<16}'. \
                      format(str(hop_time), str(ttl), str(response.src))
                # print str(hop_time)+' '+'ms', ttl, '->', response.src
                end_check = True
        else:
            print '************ Request timed out.'
if __name__ == '__main__':
    main()